import resource
import sys
import urllib.request
import PIL
from PIL import Image
from metal.utils.data_manipulation import normalize
import numpy as np
import dill
from autograd.parameter import Parameter
import random
import cv2
import os


print('resource limit ',resource.getrlimit(resource.RLIMIT_STACK))
print('recursion limit ',sys.getrecursionlimit())


def save_model(filename,model, max_rec=100000):
    max_rec = max_rec
    sys.setrecursionlimit(max_rec)
    with open(filename+'.pkl', 'wb') as file:
        dill.dump(model.layers, file)

def load_model(filename):
    with open(filename+'.pkl', 'rb') as file:
        layers = dill.load(file)
    return layers

def fetch_img(url,shape):
    I = Image.open(urllib.request.urlopen(url)).convert('L')
    I = np.asarray(I.resize((shape[-1],shape[-2]), PIL.Image.LANCZOS)).reshape(*shape)
    I = np.invert(I)
    I = normalize(I)
    return I

def _forward_pass(X, training=True, model_name=None):
    """ Calculate the output of the NN """
    layers_ = load_model(model_name)
    layer_output = Parameter(X,False)
    for layer in layers_:
        layer_output = layer.forward_pass(layer_output, training)
    return layer_output

#resize batch
#example: img_tran(X_train,8)
def imgs_trans(imgs_in,size):
    factor = size/imgs_in.shape[1]
    imgs_out = ndi.zoom(imgs_in, (1, factor, factor, 1), order=2)
    print(imgs_out.shape)
    return imgs_out

def create_training_data(img_size=50, classes=[], data_dir="", color=None):
    #classses is the name of each folder in downloads
    #data_dir is path to downloads
    #img_size for WxH

    if color == None:
        color = cv2.IMREAD_GRAYSCALE
        # TODO: add more opitons

    training_data = []
    for class_ in classes:
        path = os.path.join(data_dir, class_)
        class_num = classes.index(class_)
        for img in os.listdir(path):
            try:
                img_array = cv2.imread(os.path.join(path,img), color)
                new_array = cv2.resize(img_array,(img_size,img_size)) #bug need to fix channels
                training_data.append([new_array,class_num])
            except exception as e:
                pass
    random.shuffle(training_data)
    imgs = []
    labs = []
    for img, lab in training_data:
        imgs.append(img)
        labs.append(lab)
    imgs = np.array(imgs).reshape(-1, img_size,img_size, 1)
    return imgs, labs



import numpy as np
from autograd.tensor import Tensor
from autograd.parameter import Parameter
from metal.module import Module
from autograd.dependency import Dependency
import math
import copy
from metal.layers.activation_functions import Sigmoid, ReLU, TanH, Softmax



activation_functions = {
    'relu': ReLU,
    'sigmoid': Sigmoid,
    'tanh': TanH,
    'softmax':Softmax,
}

class Layer(Module):
    __slots__ = ('input_shape')
    """docstring for Layer."""

    def set_input_shape(self, shape):
        """ Sets the shape that the layer expects of the input in the forward
        pass method """
        self.input_shape = shape

    def layer_name(self):
        """ The name of the layer. Used in model summary. """
        return self.__class__.__name__

    def parameters_(self):
        """ The number of trainable parameters used by the layer """
        return 0

    def forward_pass(self, X, training):
        """ Propogates the signal forward in the network """
        raise NotImplementedError()

    def update_pass(self):
        """ Propogates the accumulated gradient backwards in the network.
        If the has trainable weights then these weights are also tuned in this method.
        As input (accum_grad) it receives the gradient with respect to the output of the layer and
        returns the gradient with respect to the output of the previous layer. """
        raise NotImplementedError()

    def output_shape(self):
        """ The shape of the output produced by forward_pass """
        raise NotImplementedError()



class Activation(Layer):
    __slots__=('activation_name','activation_func','trainable','layer_input')
    """A layer that applies an activation operation to the input.

    Parameters:
    -----------
    name: string
        The name of the activation function that will be used.
    """

    def __init__(self, name):
        self.activation_name = name
        self.activation_func = activation_functions[name]()
        self.trainable = True

    def layer_name(self):
        return "Activation (%s)" % (self.activation_func.__class__.__name__)

    def forward_pass(self, X, training=True):
        self.layer_input = X
        return self.activation_func(X)

    def update_pass(self):
        self.zero_grad()

    def output_shape(self):
        return self.input_shape
